#!/usr/bin/env bash
#
# Copyright (c) 2012 Avira Operations GmbH & Co. KG
#
#
# Description: Utils functions used for pre and post scripts
# VERSION=1.0.0.7

# The Avira ICAP binary directory
# NOTE: The default value for this variable is ${INSTALL_DIR}, the directory where the update is made.
#       If INSTALL_DIR is different than the Avira ICAP binary directory, then the following line should
#       be uncommented and the value replaced with the actual location of the Avira ICAP binary directory.
#
#ICAPBIN_DIR="<path_to_Avira_ICAP_binary_directory>"

# The name of the Avira ICAP binary file
ICAPBIN_NAME="av-icapd"
# The maximum timeout, in seconds, to wait for the running Avira ICAP process to stop
SHUTDOWN_TIMEOUT_SEC=60
# The file where all the Avira ICAP pids are saved
ICAPBIN_POST_UTILS_FILE="icap_post_utils.$PPID"

# Returns the PIDs of the processes having the provided name
# Input:
#    - the name of the process
#    - the binary directory of the process
# Output: array with process PIDs
function process_pid_by_name
{
    local process_name="$1"
    local binary_dir="$2"
    local pids=()

    # get the list of processes having the given name
    pids=($(ps -o pid --no-headers -C $process_name))

    # store only the processes that have the current install dir
    for ((i = 0; $i < ${#pids[@]}; i++))
    do
        # get the running directory of the process
        run_dir=$(dirname "$(readlink /proc/${pids[$i]}/exe | sed s/\(deleted\)//)")

        # print the pid that has its process started from the install dir
        if [ "${run_dir}" == "${binary_dir}" ]
        then
            echo ${pids[$i]}
        fi
    done

    return 0
}

# Restarts the Avira ICAP process based on the info of the provided pid
# Input:
#    - the pid of the process
#    - the binary path of the process
#    - the shutdown timeout in seconds
function restart_process_by_pid
{
    local pid="$1"
    local bin_path="$2"
    local shutdown_timeout_sec="$3"
    local args=\"`cut -d '' -f2- /proc/$pid/cmdline | sed 's/\x0$//' | sed 's/\x0/\" \"/g'`\"

    echo "INFO: Stopping Avira ICAP (pid $pid)"

    # stop the old Avira ICAP process
    if ! kill -INT $pid &>/dev/null
    then
        echo "ERROR: Could not send SIGINT to '$pid'"
        return 1
    fi

    # wait for the process to stop
    for ((i=0; $i < $shutdown_timeout_sec; i++))
    do
        sleep 1
        ps -p $pid &> /dev/null || break
    done

    # Check if the Avira ICAP was properly stopped
    if [ $i -eq $shutdown_timeout_sec ]
    then
        echo "ERROR: It was not possible to stop the running Avira ICAP (pid $pid) in a timely manner ($shutdown_timeout_sec seconds)."
        exit 1
    fi

    # remove "-N" or "--foreground" from the command line arguments
    args=`echo $args | sed 's/ "-N"//g'`
    args=`echo $args | sed 's/ "--foreground"//g'`

    # make args empty if it doesn't contain any arguments
    args=`echo $args | sed 's/^""$//'`

    # start the new Avira ICAP with the same cmdline arguments as the old one
    echo "INFO: Starting new Avira ICAP process: '$bin_path $args'"
    eval "\"$bin_path\" $args"

    # Check if the Avira ICAP started by verifying the return code
    if [ $? -ne 0 ]
    then
        echo "ERROR: The new Avira ICAP process did not start"
        exit 1
    fi

    return 0
}

# Reloads the Avira ICAP process based on the info of the provided pid
# Input:
#    - the pid of the process
function reload_process_by_pid
{
    local pid="$1"

    echo "INFO: Reloading Avira ICAP (pid $pid)"

    # reload the Avira ICAP process
    if ! kill -USR1 $pid &>/dev/null
    then
        echo "ERROR: Could not send SIGUSR1 to '$pid'"
        return 1
    fi

    # TODO: Check that reload was successful

    return 0
}
