#ifndef SAVAPI_H__
#define SAVAPI_H__

#include "stchar.h"

/**
 * \mainpage SAVAPI
 *
 * SAVAPI stands for Secure AntiVirus Application Programming Interface.
 * Its main purpose is to offer a very simple scanning interface for clients who want
 * to programmatically integrate scanning services into their applications.
 *
 * This document explains how to use the SAVAPI interface shared libraries written and provided
 * by Avira Operations GmbH & Co. KG.
 * The provided LICENSE file explains the terms and conditions
 * for using the SAVAPI interface shared libraries.
 *
 * By utilizing the implemented SAVAPI interface, it becomes
 * very simple to quickly integrate SAVAPI into your applications.
 * There is no need to be concerned about protocol details,
 * since they have all been implemented by Avira Operations GmbH & Co. KG.
 *
 * See: \ref defs "SAVAPI options", \ref defs "SAVAPI constants", \ref defs "SAVAPI defines",
 * \ref structs "SAVAPI structures", \ref funcs "SAVAPI functions"
 *
 */

/**
 * \defgroup defs SAVAPI constants
 * @{
 *
 * \defgroup rets SAVAPI return codes
 * @{
 */

/**
 * \brief Operation ended with success
 */
#define SAVAPI3_S_OK                                    0
/**
 * \brief One of supplied parameters is invalid
 * \note At least one of the function's parameters is invalid (invalid pointers,
 * empty strings, out of range values, etc.).
 */
#define SAVAPI3_E_INVALID_PARAMETER                     1
/**
 * \brief SAVAPI was already initialized
 * \note Trying to initialize an already initialized SAVAPI library (\ref SAVAPI3_initialize was
 * already called successfully).
 */
#define SAVAPI3_E_ALREADY_INITIALIZED                   2
/**
 * \brief SAVAPI is not initialized
 * \note The used functionality requires the SAVAPI library to be initialized first
 * (a successful call of \ref SAVAPI3_initialize is needed before).
 */
#define SAVAPI3_E_NOT_INITIALIZED                       3
/**
 * \brief Supplied buffer is too small
 * \note An interface function that requires a buffer size as parameter was called
 * with a value smaller than the needed size.
 */
#define SAVAPI3_E_BUFFER_TOO_SMALL                      4
/**
 * \brief Connection mode flag is not set
 * \note The \ref SAVAPI3_INSTANCE_INIT::flags in the instance creation structure
 * is not set to a known connection mode.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_CONNECTION_MODE_NOT_SET               5
/**
 * \brief Host name is not set
 * \note The \ref SAVAPI3_INSTANCE_INIT::host_name field in the instance creation
 * structure was not set.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_HOSTNAME_NOT_SET                      6
/**
 * \brief Memory allocation failed
 * \note There is not enough memory available for allocation.
 */
#define SAVAPI3_E_NO_MEMORY                             7
/**
 * \brief VDF file(s) not found
 * \note Path to the VDF files is not correct, files are missing, or there are
 * no access rights to open the files.
 */
#define SAVAPI3_E_VDF_NOT_FOUND                         8
/**
 * \brief VDF file(s) read failed
 * \note VDF files are damaged or truncated.
 */
#define SAVAPI3_E_VDF_READ                              9
/**
 * \brief VDF file(s) crc check failed
 * \note One ore more VDF files failed checksum check because they were damaged,
 * manipulated or truncated.
 */
#define SAVAPI3_E_VDF_CRC                               10
/**
 * \brief Inconsistent versions in VDF files set
 * \note There are incompatible VDF files within the VDF set. Not all relevant VDF
 * files were downloaded or the engine is too old for the present VDF set.
 */
#define SAVAPI3_E_VDF_VERSION                           11
/**
 * \brief Engine initialization failed
 * \note Engine is too old for this version of SAVAPI. SAVAPI used a wrong character
 * set when initializing the engine.
 */
#define SAVAPI3_E_WRONG_ENGINE                          12
/**
 * \brief Engine file(s) not found
 * \note One or more engine files are not present in the engine's directory.
 */
#define SAVAPI3_E_ENGINE_NOT_FOUND                      13
/**
 * \brief Invalid SAVAPI binary encountered
 * \details Not used.
 */
#define SAVAPI3_E_WRONG_SAVAPI                          14
/**
 * \brief Inconsistent versions in engine files set
 * \note There are incompatible engine files within the engine set which do not match the
 * expected version. The engine file set was not updated or is too old for the present engine set.
 */
#define SAVAPI3_E_SELFCHK_PATCHED                       15
/**
 * \brief Engine file(s) read failed
 * \note One or more engine files are damaged or truncated.
 */
#define SAVAPI3_E_SELFCHK_FILE_ERR                      16
/**
 * \brief Engine file(s) crc check failed
 * \note One or more engine files failed checksum check because they were manipulated,
 * damaged or truncated.
 */
#define SAVAPI3_E_SELFCHK_FILE_CRC                      17
/**
 * \brief Keyfile error
 * \note Not used.
  */
#define SAVAPI3_E_KEYFILE                               18
/**
 * \brief SAVAPI internal error
 * \note An unexpected internal event prevented the normal execution of the library
 * (incorrect pointers, incorrect return values, etc.). Normally this error should
 * never occur. If this error occurs there is a major problem which must be fixed.
 */
#define SAVAPI3_E_INTERNAL                              19
/**
 * \brief Unsupported feature
 * \note The requested functionality (feature, command, option) may be known but it is
 * not supported by this version of SAVAPI or engine. For instance a called function
 * is not available in the current library mode (\ref SAVAPI3_is_running_ex() is not
 * available in SAVAPI Library, or \ref SAVAPI3_reload_engine_ex() is not available
 * in SAVAPI Client Library); or the used signal id is unknown (the only known signal
 * for \ref SAVAPI3_send_signal is \ref SAVAPI3_SIGNAL_SCAN_ABORT); or a new functionality
 * was added but is not yet implemented or not supported yet by the current library version
 * or within the current engine version.
 */
#define SAVAPI3_E_NOT_SUPPORTED                         20
/**
 * \brief An error occurred during a file scan
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_ERROR                          21
/**
 * \brief Could not extract file
 * \note A file to extract during an archive scanning could not be found.
 */
#define SAVAPI3_E_RESULT_FILE_NOT_FOUND                 22
/**
 * \brief Option is not supported
 * \note Trying to set or retrieve a value for an option with an unknown or
 *  obsolete id (for instance \ref SAVAPI3_OPTION_UPDATE_SERVERS is obsolete).
 */
#define SAVAPI3_E_OPTION_NOT_SUPPORTED                  23
/**
 * \brief Archive maximum recursion limit reached
 * \note The limit on the maximum number of archive recursions was exceeded when extracting
 * a file because the file was packed too many times or it contained other deeply nested files.
 * The decompression will be aborted as soon as the limit is exceeded.
 */
#define SAVAPI3_E_HIT_MAX_REC                           24
/**
 * \brief Archive maximum extraction size reached
 * \note Size of an uncompressed file has exceeded the maximum extraction size. The decompression
 * will be aborted as soon as the limit is exceeded.
 */
#define SAVAPI3_E_HIT_MAX_SIZE                          25
/**
 * \brief Archive maximum extraction ratio reached
 * \note Size of an uncompressed file has exceeded the maximum extraction ratio.
 * The decompression will be aborted as soon as the limit is exceeded.
 */
#define SAVAPI3_E_HIT_MAX_RATIO                         26
/**
 * \brief Encrypted contents found
 * \note One or more files inside the archive are encrypted, but there are also
 * files which are not encrypted and can be extracted; or all files inside the
 * archive are encrypted and it's not possible to extract them.
 */
#define SAVAPI3_E_ENCRYPTED                             27
/**
 * \brief Unsupported archive type/format
 * \note The archive type is not supported. The version of a known archive type is
 * not supported. The compression method is not supported. The archive format is unknown.
 */
#define SAVAPI3_E_UNSUPPORTED                           28
/**
 * \brief Archive generic processing error
 * \note Any other archive scan processing error which is not covered by
 * SAVAPI3_E_PROC_<name> error codes.
 */
#define SAVAPI3_E_PROC_ERROR                            29
/**
 * \brief File was not completely scanned
 * \note Scanning was aborted by user or as result of a terminal warning or error.
 */
#define SAVAPI3_E_INCOMPLETE                            30
/**
 * \brief Cannot extract multi-volume archive
 * \note In case of an archive which is part of a multi-volume archive set, a file
 * could not be fully extracted because is split over several archive parts.
 * Processing the next file may be successful if all information is stored in that part.
 */
#define SAVAPI3_E_PARTIAL                               31
/**
 * \brief Maximum number of files in archive reached
 * \note Maximum files count limit was reached while scanning an archive.
 * The scanning will be aborted as soon as the limit is exceeded.
 */
#define SAVAPI3_E_HIT_MAX_COUNT                         32
/**
 * \brief Scan was aborted by signal
 * \note A scan in progress was aborted by user with \ref SAVAPI3_SIGNAL_SCAN_ABORT signal.
 */
#define SAVAPI3_E_ABORTED                               33
/**
 * \brief Scan timed out
 * \note A scan in progress exceeded the maximum user set scan time-out.
 */
#define SAVAPI3_E_TIMEOUT                               34
/**
 * \brief Possible infected file
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_SUSPICIOUS                     35
/**
 * \brief Could not decrypt virus
 * \note Not used.
 */
#define SAVAPI3_E_DECRYPT                               36
/**
 * \brief Read error (boot record access)
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_READ                           37
/**
 * \brief Write error (boot record access)
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_WRITE                          38
/**
 * \brief Invalid sector (no bios signature) (boot record access)
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_INVALID                        39
/**
 * \brief Could not open file
 * \note File is missing or there are no access rights to open it.
 */
#define SAVAPI3_E_FILE_OPEN                             40
/** File read error */
/**
 * \brief Could not read file
 * \note There are no access rights to read file, or the file has been removed, or
 * data from file end is missing, or file is truncated.
 */
#define SAVAPI3_E_FILE_READ                             41
/**
 * \brief Could not write file
 * \note There are no access rights to write file, or the file has been removed.
 * Disk quota exceeded or disk is damaged.
 */
#define SAVAPI3_E_FILE_WRITE                            42
/**
 * \brief SAVAPI in DEMO mode. Call not executed
 * \note Not used.
 */
#define SAVAPI3_E_DEMOMODE                              43
/**
 * \brief Problem while getting disk geometry
 * \note Not used.
 */
#define SAVAPI3_E_QUERY_DISK_PARAM                      44
/**
 * \brief Wrong file size in directory
 * \note Not used.
 */
#define SAVAPI3_E_FILE_LEN                              45
/**
 * \brief Invalid file date
 * \note Not used.
 */
#define SAVAPI3_E_FILE_DATE                             46
/**
 * \brief Possible corrupted file
 * \note Not used.
 */
#define SAVAPI3_E_FILE_DAMAGED                          47
/**
 * \brief Macro heuristic: possible dropper
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_DROPPER                        48
/**
 * \brief Macro heuristic: possible trojan horse
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_TROJAN                         49
/**
 * \brief Macro heuristic: possible polymorphic virus
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_POLYMORPHIC                    50
/**
 * \brief MBS is ok, force a backup to user
 * \note Not used.
 */
#define SAVAPI3_E_FORCE_BACKUP                          51
/**
 * \brief Partition tables unequal
 * \note Not used.
 */
#define SAVAPI3_E_PARTITION_TABLE                       52
/**
 * \brief File contains a boot virus image
 * \note Not used.
 */
#define SAVAPI3_E_RESULT_BOOTIMAGE                      53
/**
 * \brief File is packed PKLite or LZExe
 * \note Not used.
 */
#define SAVAPI3_E_FILE_PACKED                           54
/**
 * \brief File is a compound doc (OLE2)
 * \note Not used.
 */
#define SAVAPI3_E_FILE_OLE                              55
/**
 * \brief File contains a word template
 * \note Not used.
 */
#define SAVAPI3_E_FILE_TEMPLATE                         56
/**
 * \brief File contains macros
 * \note Not used.
 */
#define SAVAPI3_E_FILE_MACRO                            57
/**
 * \brief File is an archive
 * \note Not used.
 */
#define SAVAPI3_E_FILE_ARCHIVE                          58
/**
 * \brief Known good boot sector
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_KNOWN                          59
/**
 * \brief Unknown boot sector
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_UNKNOWN                        60
/**
 * \brief Boot sector contains constant data
 * \note Not used.
 */
#define SAVAPI3_E_SECTOR_CONSTANT                       61
/**
 * \brief SAVAPI is not up to date
 * \note Not used.
 */
#define SAVAPI3_E_NOT_UPTODATE                          62
/**
 * \brief SAVAPI product is not set
 * \note Not used.
 */
#define SAVAPI3_E_SETUP_PRODUCT                         63
/**
 * \brief No parameter given to command
 * \note Failure in SAVAPI Client Library communication with SAVAPI service resulting
 * in commands with no parameters (for instance SET or GET commands).
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_NO_PARAMETER                          64
/**
 * \brief Invalid value in configuration or command
 * \note Failure in SAVAPI Client Library communication with SAVAPI Service resulting
 * in commands with invalid values which cannot be accepted by Service (for instance
 * SET or GET commands with invalid values). The engine path given to the
 * \ref SAVAPI3_reload_engine_ex() function collides with previous engine path.
 */
#define SAVAPI3_E_INVALID_VALUE                         65
/**
 * \brief Could not change directory
 * \note Failure in SAVAPI Client Library communication with SAVAPI Service resulting
 * in an unsuccessful SET CWD command.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_CHDIR_FAILED                          66
/**
 * \brief Path is not absolute
 * \note Path to a given or required directory is not absolute (for example the path of
 * the temporary scanning directory is not absolute).
 */
#define SAVAPI3_E_NOT_ABSOLUTE_PATH                     67
/**
 * \brief Directory path does not exist
 * \note Path to a given directory does not exist (for example the path of the temporary
 * scanning directory does not exist).
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_DIR_NOT_EXISTS                        68
/**
 * \brief File was filtered from scanning
 * \note File matched a black list rule and was not scanned.
 */
#define SAVAPI3_E_MATCHED                               69
/**
 * \brief Converting failed
 * \note A string could not be converted from one encoding to another (for instance
 * a string could not be converted from SAVAPI_TCHAR to char, or in case of
 * SAVAPI Client Library  a string could not be converted from SAVAPI_TCHAR to
 * the SAVAPI Service's text mode encoding).
 */
#define SAVAPI3_E_CONVERSION_FAILED                     70
/**
 * \brief Office document found
 * \note Not used.
 */
#define SAVAPI3_E_FILE_OFFICE                           71
/**
 * \brief Filename from archive
 * \note Not used.
 */
#define SAVAPI3_E_FILE_IN_ARCHIVE                       72
/**
 * \brief Connection with the SAVAPI Service failed
 * \note The SAVAPI service is not running on the specified interface.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_CONNECTION_FAILED                     73
/**
 * \brief Failed to receive data from the SAVAPI Service
 * \note The SAVAPI service is not running anymore.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_RECEIVE_FAILED                        74
/**
 * \brief Failed to send data to the SAVAPI Service
 * \note The SAVAPI service is not running anymore.
 * \note This error can only be triggered by the SAVAPI Client Library.
 */
#define SAVAPI3_E_SEND_FAILED                           75
/**
 * \brief Invalid option value
 * \note A configuration command received a value buffer which is not acceptable as
 * a value for the associated option id (for instance it is empty).
 */
#define SAVAPI3_E_OPTION_VALUE_INVALID                  76
/**
 * \brief Repair an infected file failed
 */
#define SAVAPI3_E_REPAIR_FAILED                         77
/**
 * \brief Failed to create file
 * \note Failed to create a temporary file in the temporary scanning directory
 * because there are no access rights, or the file already exists, etc.
 */
#define SAVAPI3_E_FILE_CREATE                           78
/**
 * \brief Failed to delete file
 * \note Failed to delete a temporary file in the temporary scanning directory
 * because there are no access rights, file is locked, file does not exist anymore, etc.
 */
#define SAVAPI3_E_FILE_DELETE                           79
/**
 * \brief Failed to close file
 * \note Failed to close a temporary file in the temporary scanning directory because
 * there are no access rights, file was accidentally deleted, etc.
 */
#define SAVAPI3_E_FILE_CLOSE                            80
/**
 * \brief Unknown engine error
 * \note Engine returns an unknown error code.
 */
#define SAVAPI3_E_UNKNOWN                               81
/**
 * \brief Failed to set a detect type option
 * \note SAVAPI failed to set a detect type option (for instance
 * \ref SAVAPI3_OPTION_DETECT_ADSPY, \ref SAVAPI3_OPTION_DETECT_APPL, others)
 */
#define SAVAPI3_E_PREFIX_SET                            90
/**
 * \brief Failed to retrieve a detect type option
 * \note SAVAPI failed to retrieve a detect type option (for instance
 * \ref SAVAPI3_OPTION_DETECT_ADSPY, \ref SAVAPI3_OPTION_DETECT_APPL, others).
 */
#define SAVAPI3_E_PREFIX_GET                            91
/**
 * \brief Invalid query for SAVAPI Service
 * \note  Failure in SAVAPI Client Library communication with SAVAPI Service
 *  resulting in an unacceptable command (invalid command, syntax error).
 * \note This error can only be triggered by the SAVAPI Client Library
 */
#define SAVAPI3_E_INVALID_QUERY                         92
/**
 * \brief Keyfile has not been found
 * \note Not used.
 */
#define SAVAPI3_E_KEY_NO_KEYFILE                        101
/**
 * \brief Access to key file has been denied
 * \note Not used.
 */
#define SAVAPI3_E_KEY_ACCESS_DENIED                     102
/**
 * \brief An invalid header has been found
 * \note Not used.
 */
#define SAVAPI3_E_KEY_INVALID_HEADER                    103
/**
 * \brief Invalid keyfile version number
 * \note Not used.
 */
#define SAVAPI3_E_KEY_KEYFILE_VERSION                   104
/**
 * \brief No valid license found
 * \note Not used.
 */
#define SAVAPI3_E_KEY_NO_LICENSE                        105
/**
 * \brief Key file is invalid (invalid CRC)
 * \note Not used.
 */
#define SAVAPI3_E_KEY_FILE_INVALID                      106
/**
 * \brief Invalid license record detected
 * \note Not used.
 */
#define SAVAPI3_E_KEY_RECORD_INVALID                    107
/**
 * \brief Application is evaluation version
 * \note Not used.
 */
#define SAVAPI3_E_KEY_EVAL_VERSION                      108
/**
 * \brief Application is demo version
 * \note Not used.
 */
#define SAVAPI3_E_KEY_DEMO_VERSION                      109
/**
 * \brief Illegal (cracked) license in keyfile
 * \note Not used.
 */
#define SAVAPI3_E_KEY_ILLEGAL_LICENSE                   110
/**
 * \brief No FUP II/III license found
 * \note Not used.
 */
#define SAVAPI3_E_KEY_NO_FUP_LICENSE                    111
/**
 * \brief No FUP II/III keyfile found
 * \note Not used.
 */
#define SAVAPI3_E_KEY_NO_FUP2_KEYFILE                   112
/**
 * \brief This key has expired
 * \note Not used.
 */
#define SAVAPI3_E_KEY_EXPIRED                           113
/**
 * \brief Error reading from key file
 * \note Not used.
 */
#define SAVAPI3_E_KEY_READ                              114
/**
 * \brief Operation not allowed (license restriction)
 * \note Scan command was issued without setting a valid product id.
 */
#define SAVAPI3_E_LICENSE_RESTRICTION                   120
/**
 * \brief Error loading engine modules
 * \note SAVAPI could not load engine modules because they are not available or
 * there are no access rights.
 */
#define SAVAPI3_E_LOADING_ENGINE_MODULES                121
/**
 * \brief SAVAPI is busy
 * \note A configuration request was given during scanning a file (for instance
 * SET/GET command or callback register/unregister command).
 * \ref SAVAPI3_uninitialize was called without releasing all SAVAPI instances before.
 */
 #define SAVAPI3_E_BUSY                                  122
/**
 * \brief Encrypted mail found
 * \note While scanning an archive an encrypted mail was found.
 */
#define SAVAPI3_E_ENCRYPTED_MIME                        123
/**
 * \brief Non addressable memory location
 * \note A scan request was issued for an address that is not in the available address
 * space for the current platform. For example, on a 64 bit machine the available
 * address space is [0..MAX_INT_64].
 */
#define SAVAPI3_E_NON_ADDRESSABLE                       124
/**
 * \brief Internal memory limit reached
 * \note An engine-internal safety limit regarding memory usage of a subroutine
 * has been reached (this can i.e. be caused by excessively large dictionaries in archives).
 */
#define SAVAPI3_E_MEMORY_LIMIT                          125
/**
 * \brief Incomplete archive block read
 * \note An archive block is damaged and could not be read.
 */
#define SAVAPI3_E_PROC_INCOMPLETE_BLOCK_READ            150
/**
 * \brief Bad archive header
 * \note The archive header is invalid.
 */
#define SAVAPI3_E_PROC_BAD_HEADER                       151
/**
 * \brief Bad compressed data
 * \note The compressed data from the archive is invalid. Some files could not be
 * extracted and scanned.
 */
#define SAVAPI3_E_PROC_INVALID_COMPRESSED_DATA          152
/**
 * \brief Obsolete archive information
 * \note Archive is packed with a very old or a developer version of a packer
 * application and contains obsolete information and unsupported entries.
 */
#define SAVAPI3_E_PROC_OBSOLETE                         153
/**
 * \brief Bad header format
 * \note The archive header has been changed with a newer (unsupported) version of
 * a packer application. The archive header is damaged.
 */
#define SAVAPI3_E_PROC_BAD_FORMAT                       154
/**
 * \brief Bad header crc
 * \note An archive header failed checksum check.
 */
#define SAVAPI3_E_PROC_HEADER_CRC                       155
/**
 * \brief Bad data crc.
 * \note Checksum of compressed data does not match.
 */
#define SAVAPI3_E_PROC_DATA_CRC                         156
/**
 * \brief Bad crc for extracted file
 * \note Checksum of a decompressed file does not match.
 */
#define SAVAPI3_E_PROC_FILE_CRC                         157
/**
 * \brief Invalid decompression table
 * \note Archive contains an invalid decompression table.
 */
#define SAVAPI3_E_PROC_BAD_TABLE                        158
/**
 * \brief Unexpected end of file
 * \note Decompression aborted because of unexpected end of file in archive.
 */
#define SAVAPI3_E_PROC_UNEXPECTED_EOF                   159
/**
 * \brief Archive internal handle error
 * \note An internal handle related to archive processing is invalid or not initialized.
 */
#define SAVAPI3_E_PROC_ARCHIVE_HANDLE                   160
/**
 * \brief No files could be extracted
 * \note Archive is invalid, corrupt or damaged.
 */
#define SAVAPI3_E_PROC_NO_FILES_TO_EXTRACT              161
/**
 * \brief Archive internal callback error
 * \note Decompression aborted because an internal archive callback is invalid or caused an error.
 */
#define SAVAPI3_E_PROC_CALLBACK                         162
/**
 * \brief File extraction failed.
 * \note Not all archive contents could be extracted.
 */
#define SAVAPI3_E_PROC_TOTAL_LOSS                       163

/**
 * @}
 * \ingroup defs
 * \defgroup error_cat Error categories
 * \note Used by the error callbacks to categorize the errors they return
 * @{
 */
/** i/o error category */
#define SAVAPI3_ECAT_ERROR_IO                           0
/** scan error category */
#define SAVAPI3_ECAT_ERROR_SCAN                         1
/** unpack error category */
#define SAVAPI3_ECAT_ERROR_UNPACK                       2
/** uncategorised error category */
#define SAVAPI3_ECAT_ERROR_GENERIC                      3
/**
 * @}
 * \ingroup defs
 * \defgroup error_level Error levels
 * \note Used by the error callbacks to categorize the returned errors
 * @{
 */
/** error level */
#define SAVAPI3_ELEVEL_ERROR                            0
/** warning level */
#define SAVAPI3_ELEVEL_WARNING                          1
/** info level */
#define SAVAPI3_ELEVEL_INFO                             2

/**
 * @}
 * \ingroup defs
 * \defgroup init_flags Initialization flags
 * \note Initialization flags will be extended on the fly when needed!
 *       The SAVAPI3_FLAG_USE_TCP and SAVAPI3_FLAG_USE_LOCAL_SOCKET must not be set
 *       simultaneously
 * @{
 */
/** TCP sockets will be used for communication (SAVAPI client-mode only) */
#define SAVAPI3_FLAG_USE_TCP                    1
/** local sockets will be used for communication (SAVAPI client-mode only) */
#define SAVAPI3_FLAG_USE_LOCAL_SOCKET           2
/** @}*/

/**
 * \ingroup defs
 * \defgroup scan_warnings Scan warnings
 * \note Warnings that can be received during the scanning process
 * @{
 */
/** File has potentially been damaged by virus */
#define SAVAPI3_W_DAMAGED                       1
/** OLE-File is potentially damaged */
#define SAVAPI3_W_OLE_DAMAGED                   2
/** File is suspicious */
#define SAVAPI3_W_SUSPICIOUS                    4
/** An abort was triggered by the progress callback */
#define SAVAPI3_W_PROGRESS_ABORT                8
/** A malformed archive header was detected */
#define SAVAPI3_W_HEADER_MALFORMED              16
/**
 * This file could be an archive bomb, ratio
 * might be exceeded or something else might
 * have happened to trigger that detection
 */
#define SAVAPI3_W_POTENTIAL_ARCH_BOMB           32
/** The ratio set by the application regarding unpacking size in archives has been exceeded */
#define SAVAPI3_W_RATIO_EXCEEDED                64
/** Unpacking has reached the maximum limit of extracted data */
#define SAVAPI3_W_MAX_EXTRACTED                 128

/** @}*/

/**
 * \ingroup defs
 * \defgroup iframes_info Iframes informations
 * \note Informations that can be received during the scanning process
 * @{
 */
/** The object is invisible to the user surfing the site */
#define SAVAPI3_HTML_CONTENT_ATTRIB_INVISIBLE  1
/** The object is very small and as such almost invisible to the user surfing the site */
#define SAVAPI3_HTML_CONTENT_ATTRIB_EXTRASMALL 2
/** The object is inserted at a very uncommon position in the HTML code */
#define SAVAPI3_HTML_CONTENT_ATTRIB_ODDPOS     4
/** The object is likely of a malicious nature */
#define SAVAPI3_HTML_CONTENT_ATTRIB_MALICIOUS  8
/** @}*/

/**
 * \ingroup defs
 * \defgroup scan_info Scan informations
 * \note Informations that can be received during the scanning process
 * @{
*/
/** OLE: file is a compound doc (OLE2) */
#define SAVAPI3_I_OLEFILE                       1
/** OLE: contains a word template */
#define SAVAPI3_I_TEMPLATE                      2
/** OLE: contains macros */
#define SAVAPI3_I_MACROS_PRESENT                4
/** OLE: all macros were deleted */
#define SAVAPI3_I_ALL_MACROS_DELETED            8
/** OLE: encrypted marker, only for DOCs */
#define SAVAPI3_I_OLE_ENCRYPTED                 16
/** SCRIPT: html contains active content (JS/VBS, etc.) */
#define SAVAPI3_I_ACTIVE_CONTENT_PRESENT        32
/** ARCHIVE: Mailbox detected */
#define SAVAPI3_I_MAILBOX                       64
/** @}*/

/**
 * \ingroup defs
 * \defgroup options SAVAPI options
 *
 * @{
 *
 * \remark
 * Almost each option used to configure the SAVAPI instance (the paths to the temporary
 * folders, the scanning options) has a default value that is written in its description
 * as a note (i.e. Default value: \<value\>).
 * \remark
 * In client-mode, the default values are dependent to the configuration used to start
 * the SAVAPI service, so the provided defaults only applies in library-mode!!!
 * \remark
 * The options that has no default (i.e. unsupported options, obsolete, ignored) will be marked
 * with the "Default value: None" string.
 *
 *
 * \defgroup get_set GET/SET options (read/write)
 * \brief "SET" requests are available to configure SAVAPI. For the following requests,
 * a "GET" counterpart is also available and these are therefore labeled as
 * "read/write". Only the "SET" version is listed here although a "GET" version
 * also exists. The "GET" response will return the same data that is provided
 * with the "SET" request (although the representation of the data may be
 * different. For example, a "SET" request with "10K" could lead to a "GET"
 * response with "10240".)
 * @{
 */

/**
 * \brief Specifies current working directory for SAVAPI.
 * \note This eliminates the need to specify full paths in filenames.
 * \note Available only in client-mode.
 * \note Default value: None
 */
#define SAVAPI3_OPTION_CWD                              1
/**
 * \brief Specifies the configuration file that is used.
 * \note The configuration file will be (re-)read as part of this request.
 * \note Available only in client-mode.
 * \note Default value: None
 */
#define SAVAPI3_OPTION_CONF                             2
/**
 * \brief Activates archive detection and scanning.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_ARCHIVE_SCAN                     3

/**
 * \brief Set the maximum allowed size (in bytes) for any file within an archive.
 * \note A value of "0" means the maximum allowed value (INT64_MAX bytes).
 * \note This setting has no meaning if ARCHIVE_SCAN is deactivated.
 * \note Default value: 1073741824 (1G)
 */
#define SAVAPI3_OPTION_ARCHIVE_MAX_SIZE                 4
/**
 * \brief Set the maximum allowed recursion within an archive.
 * \note A value of "0" means the maximum allowed value (1000 recursion levels).
 * \note This setting has no meaning if ARCHIVE_SCAN is deactivated.
 * \note Default value: 200
 */
#define SAVAPI3_OPTION_ARCHIVE_MAX_REC                  5
/**
 * \brief Set the maximum allowed decompressing-ratio within an archive.
 * \note A value of "0" means the maximum allowed value (INT32_MAX).
 * \note This setting has no meaning if ARCHIVE_SCAN is deactivated.
 * \note Default value: 150
 */
#define SAVAPI3_OPTION_ARCHIVE_MAX_RATIO                6
/**
 * \brief Set the maximum allowed number of files within an archive.
 * \note A value of "0" means the maximum allowed value (INT64_MAX).
 * \note This setting has no meaning if ARCHIVE_SCAN is deactivated.
 * \note Default value: 0
 */
#define SAVAPI3_OPTION_ARCHIVE_MAX_COUNT                7
/**
 * \brief Activates detection and scanning of mailboxes.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_MAILBOX_SCAN                     8
/**
 * \brief Activates heuristic macro detection
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_HEUR_MACRO                       9
/**
 * \brief Set the heuristic level for the engine. The available levels are:
 * - 0 - Disable heuristic detection.
 * - 1 - Lazy heuristic detection. This is the lowest possible mode, detection
 *     is not very good, but the false positives number will be low.
 * - 2 - Normal heuristic detection.
 * - 3 - High heuristic detection. This is the highest possible mode, but the false
 *     positives number will be high.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_HEUR_LEVEL                       10
/**
 * \brief Set the temporary directory used for scanning files.
 * \note SAVAPI may use other temporary directories for files that are not being scanned. These
 *       other directories can be specified with command-line arguments or in a configuration file.
 * \note Default value: The system temporary folder
 */
#define SAVAPI3_OPTION_SCAN_TEMP                        11
/**
 * \brief Set the maximum number of seconds allowed to scan a file before aborting
 * \note Default value: 0 (no timeout)
 */
#define SAVAPI3_OPTION_SCAN_TIMEOUT                     12
/**
 * \brief Activates the repairing of infected files
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_REPAIR                           13
/**
 * \brief Activates the notification of reparable infected files
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_NOTIFY_REPAIR                    14
/**
 * \brief Activates the detection of office documents
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_NOTIFY_OFFICE                    15
/**
 * \brief Activates the detection of macros within office documents
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_NOTIFY_OFFICE_MACRO              16
/**
 * \brief Specify the list of update servers.
 * \note This option is obsolete and is kept only for backward compatibility.
 * \note Trying to use this option will result in \ref SAVAPI3_E_OPTION_NOT_SUPPORTED error.
 * \note Default value: None
 */
#define SAVAPI3_OPTION_UPDATE_SERVERS                   17
/**
 * \brief Activate usage of a proxy server for updates.
 * \note This option is obsolete and is kept only for backward compatibility.
 * \note Trying to use this option will result in \ref SAVAPI3_E_OPTION_NOT_SUPPORTED error.
 * \note Default value: None
 */
#define SAVAPI3_OPTION_UPDATE_PROXY                     18
/**
 * \brief Specify the settings for a proxy server
 * \note This option is obsolete and is kept only for backward compatibility.
 * \note Trying to use this option will result in \ref SAVAPI3_E_OPTION_NOT_SUPPORTED error.
 * \note Default value: None
 */
#define SAVAPI3_OPTION_UPDATE_PROXY_SETTINGS            19
/**
 * \brief Activates the notification of virus description url
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_NOTIFY_ALERTURL                  20
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_ADSPY                     21
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_APPL                      22
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_BDC                       23
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_DIAL                      24
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_GAME                      25
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_HIDDENEXT                 26
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_JOKE                      27
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_PCK                       28
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_PHISH                     29
/**
 * \brief Activate detection for the specified type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_SPR                       30
/**
 * \brief Activate IFRAME detection
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_IFRAMES_URL                      31
/**
 * \brief Activate reporting of encrypted mails
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_REPORT_ENCRYPTED_MIME            32
/**
 * \brief Set the scanning method. Available options are:
 * - SMART   - Smart Extensions scan mode. The files scanned for malware are chosen by SAVAPI
 *             The choice is made based on the files content. This is the recommended setting.
 * - ALL     - All scan mode. Files are scanned for malware, no matter their content or extension.
 * - EXTLIST - Extensions List scan mode. Only files with specific extensions are scanned for
 *             malware content.
 * \note Default value: SMART
 */
#define SAVAPI3_OPTION_SCAN_MODE                        33
/**
 * \brief Activate detection and scanning of mails
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_MIME_SCAN                        34

/**
 * \brief Activate scanning and reporting of PGP encrypted files
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_PGP_SCAN                         35

/**
 * \brief Activate regular interval messages during scanning to confirm that the
 * SAVAPI service is still alive
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_SCAN_PROGRESS                    36

/**
 * \brief Activate detection for the ADWARE type.
 * \note Default value: 1 (enabled)
 */
#define SAVAPI3_OPTION_DETECT_ADWARE                    37

/**
 * \brief Activate detection for the PFS (possible fraudulent software) type.
 * \note Default value: 0 (disabled)
 */
#define SAVAPI3_OPTION_DETECT_PFS                       38

/** @} */
/**
 * \ingroup options
 * \defgroup set SET options (write only)
 * \brief "SET" requests are available to configure SAVAPI. Usually a "SET" request also
 * has a "GET" request counterpart to retrieve current settings. However, the
 * following commands do not have a "GET" counterpart and are therefore labeled
 * as "write only".
 * @{
 */

/**
 * \brief Set the key-id that is required by the application.
 * \note SAVAPI will check if the key-id is within the license and that it is not
 * expired. If it is available and is valid, the application is free to use SAVAPI.
 * If not, most requests will result in an error response.
 */
#define SAVAPI3_OPTION_PRODUCT                          40
/** Activate detection for all types. */
#define SAVAPI3_OPTION_DETECT_ALLTYPES                  41
/** @} */
/**
 * \ingroup options
 * \defgroup get GET options (read only)
 * \brief "GET" requests are available to retrieve current SAVAPI settings.
 * Usually a "GET" request also has a "SET" request counterpart to configure
 * SAVAPI. However, the following commands do not have a "SET" counterpart
 * and are therefore labeled as "read only".
 * @{
 */

/**
 * \brief Retrieves SAVAPI Service version for client-mode, or,
 * for library-mode, it retrieves the SAVAPI library version
 */
#define SAVAPI3_OPTION_SAVAPI                           50
/** Retrieve engine version number */
#define SAVAPI3_OPTION_AVE_VERSION                      51
/** Retrieve vdf(-set) version number */
#define SAVAPI3_OPTION_VDF_VERSION                      52
/**
 * \brief Retrieve the process-id for the SAVAPI process that is currently handling the TCP/IP connection.
 * \note Available only in client-mode.
 */
#define SAVAPI3_OPTION_PID                              53
/** Retrieve the expiration date of the SAVAPI license (YYYYMMDD) */
#define SAVAPI3_OPTION_EXPIRE                           54
/** Retrieve the number of signatures in the vdf(-set) */
#define SAVAPI3_OPTION_VDFSIGCOUNT                      55
/**
 * \brief Retrieve the various types that can be detected (and dynamically turned on/off).
 * \note The types are returned as a comma separated list. The current value (Jan 2006)
 *       would be: ADSPY,BDC,DIAL,GAME,HEUR_DBLEXT,JOKE,PCK,SPR
 */
#define SAVAPI3_OPTION_SELECTABLE_DETECT                56
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_ADSPY                      57
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_APPL                       58
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_BDC                        59
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_DIAL                       60
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_GAME                       61
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_HIDDENEXT                  62
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_JOKE                       63
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_PCK                        64
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_PHISH                      65
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_SPR                        66
/** Retrieve the license date of the vdf(-set). Date is the form of YYYYMMDD. */
#define SAVAPI3_OPTION_VDF_DATE                         67
/**
 * \brief Retrieve the path to the file containing the dump of the malware names
 * \note This option's value should only be checked after a successfully call to the
 *       \ref SAVAPI3_extract_malware_names function, otherwise it will contain
 *       an empty string.
 * \note Default value: None
 * \note This option is available starting with SAVAPI Library 1.4.0.1 version
 */
#define SAVAPI3_OPTION_MALWARE_NAMES_FILE               68

/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_ADWARE                     69
/** Retrieve the english description for the given type.*/
#define SAVAPI3_OPTION_DESCR_PFS                        70

/** @} */
/** @}*/

/**
 * \ingroup defs
 * \defgroup callbacks_id Callbacks' ids
 * @{
 */

/**
 * \brief Triggered after a file is scanned. The callback data contains the status of the last scanned file.
 */
#define SAVAPI3_CALLBACK_REPORT_FILE_STATUS             0

/**
 * \brief Triggered to report an error or a warning.
 * \note Can be triggered at any time.
 */
#define SAVAPI3_CALLBACK_REPORT_ERROR                   3

/**
 * \brief Triggered before the scanning begins. Can be used to create filters.  For example, if we want to scan only .exe files, we install
 * a PRE_SCAN callback. Before each file is scanned, the PRE_SCAN callback will be called. Inside our implementation of the callback,
 * we implement the filter. If the returned code is success, the file will be scanned, otherwise it will be skipped.
 * \note Currently, this callback is not triggered in client-mode.
 */
#define SAVAPI3_CALLBACK_PRE_SCAN                       4

/**
 * \brief Triggered before opening an archive. If the returned code is success, the archive will be opened, otherwise
 * it will be skipped from opening.
 * \note Currently, this callback is not triggered in client-mode.
 */
#define SAVAPI3_CALLBACK_ARCHIVE_OPEN                   5

/**
 * \brief Triggered when messages related to scan progress are available.
 */
#define SAVAPI3_CALLBACK_PROGRESS_REPORT                6

/**
 * \brief Triggered when messages related to scan (progress, warnings or infos that are not error_callback related)
 * are available. IFRAME detection (\ref SAVAPI3_OPTION_IFRAMES_URL) will be reported through this callback.
 */
#define SAVAPI3_CALLBACK_CONTENT_REPORT                 7

/**
 * \brief Triggered when messages related to scan process details are available.
 * The virus description url (\ref SAVAPI3_OPTION_NOTIFY_ALERTURL)
 */
#define SAVAPI3_CALLBACK_SCAN_DETAILS_REPORT            8

/** @}
 * \ingroup defs
 * \defgroup report_detail_types SAVAPI report scan details types
 * @{
 */

/** Malware URL description */
#define SAVAPI3_REPORT_ALERTURL                 1
/** Malware found in the object can be repaired */
#define SAVAPI3_REPORT_REPAIRABLE               2

/** @}
 * \ingroup defs
 * \defgroup report_content_types SAVAPI report content types
 * @{
 */

/** IFRAME URL report */
#define SAVAPI3_REPORT_CONTENT_IFRAME                   0

/** @}
 * \ingroup defs
 * \defgroup signal_ids SAVAPI signals
 * @{
 */

/**
 * \brief Will cause the SAVAPI instance to abort scanning process as soon as possible
 * \note The signal have no associated specific data. When calling \ref SAVAPI3_send_signal function, "data"
 *       argument may be NULL.
 * \todo Add new signals as needed.
 */
#define SAVAPI3_SIGNAL_SCAN_ABORT                       1

/** @}
 * \ingroup defs
 * \defgroup other_commands_ids SAVAPI commands
 * @{
 */

/**
 * \brief Synchronous commands for the SAVAPI client-mode.
 * \note The following commands are obsolete and are kept only for backward compatibility.
 */
/** Reload SAVAPI original configuration files */
#define SAVAPI3_COMMAND_RELOAD                  0
/** Shutdown SAVAPI */
#define SAVAPI3_COMMAND_SHUTDOWN                2
/** Test if SAVAPI daemon is alive */
#define SAVAPI3_COMMAND_PING                    3
/** Check for updates */
#define SAVAPI3_COMMAND_UPDATE_CHECK            4
/** Perform the update if available */
#define SAVAPI3_COMMAND_UPDATE_START            5

/**
 * @}
 * \ingroup defs
 * \defgroup scan_statuses SAVAPI scan statuses
 * @{
 */
/** Processed object is clean. */
#define SAVAPI3_SCAN_STATUS_CLEAN               0
/** Viral code found during object processing. */
#define SAVAPI3_SCAN_STATUS_INFECTED            1
/** Suspicious code found during object processing. */
#define SAVAPI3_SCAN_STATUS_SUSPICIOUS          2
/** An error occurred during object processing */
#define SAVAPI3_SCAN_STATUS_ERROR               3
/** Object processing finished. */
#define SAVAPI3_SCAN_STATUS_FINISHED            4


/** @} */

/**
 * @}
 * \ingroup defs
 * \defgroup file_type File types
 * \note Used by the callbacks to report the type of the scanned file
 * @{
 */
/** Regular file (all files are regular) */
#define SAVAPI3_FTYPE_REGULAR                           4
/** Known archive type */
#define SAVAPI3_FTYPE_ARCHIVE                           1
/** File is in an archive */
#define SAVAPI3_FTYPE_IN_ARCHIVE                        2
/** @} */
/** @} */

/**
 * @}
 * \ingroup defs
 * \defgroup filename_flags Filename flags
 * \note Used by the callbacks to report the state of the last reported filename
 * @{
 */
/**
 * \brief The last filename is not the one reported by the engine, but a default one set by the lib
 * \note This flag is passed to the callback data structures: \ref SAVAPI3_PRESCAN_DATA,
 *       \ref SAVAPI3_ARCHIVE_OPEN_DATA, \ref SAVAPI3_FILE_STATUS_DATA in the 'flags' field.
 * \note This is available starting with 1.4.0.13 Library version.
 */
#define SAVAPI3_FLAG_LAST_FILENAME_DEFAULT 1 << 0
/** @} */

/**
 * \defgroup structs SAVAPI structures
 * @{
 */

/**
 * \brief The structure used at SAVAPI initialization
 */
typedef struct SAVAPI3_global_init
{
    /** The unique program number which identifies the 3rd party application
    * for the license checking function.
    * This has to be requested from Avira.
    */
    unsigned int    program_type;
    /** Path to a directory containing engine modules */
    SAVAPI_TCHAR    *engine_dirpath;
    /** Path to a directory containing the signature files */
    SAVAPI_TCHAR    *vdfs_dirpath;

    /* Start of Backward compatibility options - These options have no function
    anymore and will be ignored if further defined. */

    /** IGNORED OPTION - Path to a directory containing avll license library */
    SAVAPI_TCHAR    *avll_dirpath;

    /* End of Backward compatibility options */

    /** Full path to a license file.
     * \note If specified, the \a key_dir parameter will be ignored
     */
    SAVAPI_TCHAR    *key_file_name;

    /** Full path to a folder containing one or more license file(s) with extension '.key'
     * \note The parameter is ignored if \a key_file_name parameter is not-NULL
     */
    SAVAPI_TCHAR    *key_dir;
} SAVAPI3_GLOBAL_INIT;

/**
 * \brief The structure used at SAVAPI instance creation
 */
typedef struct SAVAPI3_instance_init
{
    /** Initialization flags, right now only the flag deciding the connection type is defined.
    * See \ref init_flags
    */
    unsigned int flags;
    /** Specified the connection timeout in milliseconds.
    * \note Available only in client-mode.
    */
    unsigned int connection_timeout;
    /** Specifies the machine on which the SAVAPI daemon is located.
    * \note Used only in client-mode.
    */
    SAVAPI_TCHAR *host_name;
    /** Specifies the port on which to connect to the daemon.
    * \note Used in the same conditions as \ref host_name.
    */
    unsigned int port;
    /** Specifies the timeout (in milliseconds) of the scan operation.
    * \note Used only in client-mode.
    */
    unsigned int scan_timeout;
    /** Specifies the timeout (in milliseconds) of the get options operation.
    * \note Used only in client-mode.
    */
    unsigned int get_timeout;
    /** Specifies the timeout (in milliseconds) of the set options operation.
    * \note Used only in client-mode.
    */
    unsigned int set_timeout;
    /** This field is not used.
    * \note Any data contained by this field will be ignored.
    * \warning Please note that this field will be removed in a future release.
    */
    SAVAPI_TCHAR *username;
    /** This field is not used.
    * \note Any data contained by this field will be ignored.
    * \warning Please note that this field will be removed in a future release.
    */
    SAVAPI_TCHAR *password;
} SAVAPI3_INSTANCE_INIT;


/**
 * \brief Contains data about the scanned file
 */
typedef struct SAVAPI3_file_info
{
    /** file name */
    SAVAPI_TCHAR    *name;
    /** The file type. See \ref file_type.
    * Note: can be one or more types (ex: file is an archive and is found in an archive).
    */
    unsigned int    type;
    /** The file recursion level (0 for regular files, +1 for each level in an archive) */
    unsigned int    level;
} SAVAPI3_FILE_INFO;

/**
 * \brief Contains data about the found malware in an infected/suspicious file
 */
typedef struct SAVAPI3_malware_info
{
    /** The malware name or null if file is clean */
    SAVAPI_TCHAR    *name;
    /** Kind of the found malware */
    SAVAPI_TCHAR    *type;
    /** Additional information about found malware */
    SAVAPI_TCHAR    *message;
    /** mallware flags */
    SAVAPI_TCHAR    *app_flags;
    /** 1 if malware removable/ 0 otherwise */
    unsigned int    removable;
    /** Malware signature found at correct offset */
    unsigned short  strict;
} SAVAPI3_MALWARE_INFO;


/**
 * \brief Contains the data sent to a prescan callback
 */
typedef struct SAVAPI3_pre_scan_data
{
    /** General purpose flags field. \note Currently defined flags: see @ref filename_flags group */
    unsigned int            flags;
    /** Information (name, type, level) about the scanned file */
    SAVAPI3_FILE_INFO       file_info;
} SAVAPI3_PRESCAN_DATA;

/**
 * \brief Contains the data sent to a archive_open callback
 */
typedef struct SAVAPI3_archive_open_data
{
    /** General purpose flags field. \note Currently defined flags: see @ref filename_flags group. */
    unsigned int            flags;
    /** Information (name, type, level) about the archive to be opened */
    SAVAPI3_FILE_INFO       file_info;
} SAVAPI3_ARCHIVE_OPEN_DATA;

/**
 * \brief Generic container
 *
 * This kind of container is very useful in case of need to store many options. It offers a very elegant
 * encapsulation and a very high flexibility (the user will not know how data will be stored).\n
 * The elements from container are accessed using a key (SAVAPI3_key_value#id member). The element is accessed through
 * SAVAPI3_key_value#value member and its type through SAVAPI3_key_value#type member
 */
typedef struct SAVAPI3_key_value
{
    /** The element associated id */
    unsigned int id;
    /** The element type */
    unsigned int type;
    /** The element value */
    char *value;
} SAVAPI3_KEY_VALUE;

/**
 * \brief Contains the data sent to a report file status callback
 * \note See \ref SAVAPI3_CALLBACK_REPORT_FILE_STATUS
 */
typedef struct SAVAPI3_file_status_data
{
    /** General purpose flags field. \note Currently defined flags: check @ref filename_flags group */
    unsigned int flags;
    /** File scan answer. See \ref scan_statuses for available values */
    unsigned int scan_answer;
    /** File information (name, type, level). See \ref SAVAPI3_file_info for more details */
    SAVAPI3_FILE_INFO file_info;
    /**
    * \brief Malware information (name, type, etc).
    * See \ref SAVAPI3_malware_info for more details.
    * \note Contains data only if the object processed is not clean.
    */
    SAVAPI3_MALWARE_INFO malware_info;
    /** warning value to report
    * See \ref scan_warnings
    */
    unsigned int warning;
    /** additional info to report
    * See \ref scan_info
    */
    unsigned int info;
} SAVAPI3_FILE_STATUS_DATA;


/**
 * \brief The structure associated with report error callback
 *
 * The callback is triggered each time an error occurred on scanning process
 * (an I/O error for instance). Also the callback can be called if warnings or infos
 * reports during scanning are activated.
 *
 * \note See \ref SAVAPI3_CALLBACK_REPORT_ERROR
 */
typedef struct SAVAPI3_error_data
{
    /** Information (name, type, level) about the file where the error occured */
    SAVAPI3_FILE_INFO file_info;
    /** error category see \ref error_cat */
    unsigned int category;
    /** error level see \ref error_level */
    unsigned int level;
    /** error code. See \ref rets
    * \note If error level is not SAVAPI3_ELEVEL_ERROR this field contains flags.
    * See \ref scan_warnings and \ref scan_info
    */
    int code;
    /** The container contain currently only the error code string.
    * \note The \ref code is the id within container
    */
    SAVAPI3_KEY_VALUE *options;
} SAVAPI3_ERROR_DATA;

/**
 * \brief The structure associated with report progress callback
 */
typedef struct SAVAPI3_report_progress_data
{
    /** Reserved */
    unsigned int    flags;
    /** the progress message */
    SAVAPI_TCHAR    *message;
} SAVAPI3_REPORT_PROGRESS_DATA;

/**
 * \brief Structure associated with the iframe report
 */
typedef struct SAVAPI3_iframe_url_data
{
    /** iframe attribute. See \ref iframes_info */
    unsigned int    attribute;
    /** iframe url. */
    SAVAPI_TCHAR    *url;
} SAVAPI3_IFRAME_URL_DATA;

/**
 * \brief The structure associated with report content callback
 */
typedef struct SAVAPI3_report_content_data
{
    /** Reserved */
    unsigned int            flags;
    /** report type (\ref report_content_types) */
    unsigned int            type;
    /** Information (name, type, level) about the scanned file */
    SAVAPI3_FILE_INFO       file_info;
    /** Union used to switch the content data depending on the received 'type' */
    union _content_data
    {
        /** Information about the iframe report (url, attribute) */
        SAVAPI3_IFRAME_URL_DATA *iframeurl_data;
    } content_data;

} SAVAPI3_REPORT_CONTENT_DATA;

/**
 * \brief Structure associated with the ALERTURL report
 */
typedef struct SAVAPI3_alert_url_data
{
    /** Pointer to the string containing the alert URL */
    SAVAPI_TCHAR            *alert_url;
    /** Information (name, type, level) about the scanned file */
    SAVAPI3_FILE_INFO       file_info;
} SAVAPI3_ALERT_URL_DATA;

/**
 * \brief Structure associated with the REPAIRABLE report
 */
typedef struct SAVAPI3_repairable_data
{
    /** Information (name, type, level) about the scanned file */
    SAVAPI3_FILE_INFO       file_info;
    /** Malware information (name, type, etc). */
    SAVAPI3_MALWARE_INFO    malware_info;
} SAVAPI3_REPAIRABLE_DATA;

/**
 * \brief The structure associated with report scan details callback
 */
typedef struct SAVAPI3_report_scan_details_data
{
    /** Reserved */
    unsigned int                    flags;
    /** report type(see \ref report_detail_types) */
    unsigned int                    type;
    /** Union used to switch the scan details data depending on the received 'type' */
    union _scan_details_data
    {
        /** Contains the alert URL report */
        SAVAPI3_ALERT_URL_DATA  *alert_url_data;
        /** Contains the details about the reparable data */
        SAVAPI3_REPAIRABLE_DATA *repairable_data;
    } scan_details_data;
} SAVAPI3_REPORT_SCAN_DETAILS_DATA;

/**
 *      \brief Structure passed by SAVAPI to a user defined callback, containing all the necessary data.
 */
typedef struct SAVAPI3_callback_data
{
    /** The callback id. See \ref callbacks_id */
    unsigned int type;
    /** The callback version */
    unsigned int version;
    /** Reserved */
    unsigned int flags;
    /**
     * \brief User custom data
     * \note SAVAPI will not make any assumption regarding this field. It will just be passed back to callback
     *       function
     */
        void            *user_data;
    /**
     * \brief  Callbacks specific data
     */
    union specific_data
    {
        /** specific data for pre scan callback \n See \ref SAVAPI3_pre_scan_data */
        SAVAPI3_PRESCAN_DATA                    *pre_scan_data;
        /** specific data for archive open callback \n See \ref SAVAPI3_archive_open_data */
        SAVAPI3_ARCHIVE_OPEN_DATA               *archive_open_data;
        /** specific data for file status callback \n See \ref SAVAPI3_file_status_data */
        SAVAPI3_FILE_STATUS_DATA                *file_status_data;
        /** specific data for error report callback \n See \ref SAVAPI3_error_data */
        SAVAPI3_ERROR_DATA                      *error_data;
        /** specific data for report progress callback \n See \ref SAVAPI3_report_progress_data */
        SAVAPI3_REPORT_PROGRESS_DATA            *report_progress_data;
        /** specific data for report content callback. See \ref SAVAPI3_report_content_data */
        SAVAPI3_REPORT_CONTENT_DATA             *report_content_data;
        /** specific data for report scan details callback. See \ref SAVAPI3_report_scan_details_data */
        SAVAPI3_REPORT_SCAN_DETAILS_DATA        *report_scan_details_data;
        /** private data. Reserved for internal use */
        void                                    *private_data;
    } callback_data;
} SAVAPI3_CALLBACK_DATA;

/**
 * \brief  The structure to be passed when sending a signal
 */
typedef struct SAVAPI3_signal_data
{
    /** signal id. See \ref signal_ids */
    unsigned int signal_id;

    /**
     * \brief Signal specific data
     * \note Currently SAVAPI has defined only \ref SAVAPI3_SIGNAL_SCAN_ABORT signal which doesn't require any
     *       data. Thus, "specific_data" field is currently empty.
     * \todo Add specific date as soon as new signals, which require data will be defined.
    */
    void *signal_data;
} SAVAPI3_SIGNAL_DATA;

/**
 * \brief  The structure to be passed when sending a command
 */
typedef struct SAVAPI3_command_data
{
    /** signal id. See \ref signal_ids */
    unsigned int signal_id;

    /**
     * \brief Signal specific data
     * \note Currently SAVAPI has defined only \ref SAVAPI3_SIGNAL_SCAN_ABORT signal which doesn't require any
     *       data. Thus, "specific_data" field is currently empty.
     * \todo Add specific date as soon as new signals, which require data will be defined.
    */
    /*      union signal_specific_data
    {
    } signal_data;
    */
    void *command_data;
} SAVAPI3_COMMAND_DATA;

/**
 * \brief The structure used to retrieve SAVAPI version
 */
typedef struct SAVAPI3_version
{
    /** Major version of the product */
    unsigned int major;
    /** Minor version of the product */
    unsigned int minor;
    /** Major version of the build */
    unsigned int build_major;
    /** Minor version of the build */
    unsigned int build_minor;
} SAVAPI3_VERSION;

/**
 * \brief The enumeration used to specify the SAVAPI's logging levels
 */
typedef enum _SAVAPI3_log_level
{
    /** Low level (debug, trace) messages.
     * This the service MESSAGE level equivalent
     */
    SAVAPI3_LOG_DEBUG = 0,
    /** informative messages */
    SAVAPI3_LOG_INFO,
    /** warning messages */
    SAVAPI3_LOG_WARNING,
    /** alert messages (i.e. malware found or any other alert) */
    SAVAPI3_LOG_ALERT,
    /** error messages */
    SAVAPI3_LOG_ERROR
} SAVAPI3_LOG_LEVEL;

/**
 * \brief Defines the type of an engine module
 */
typedef enum SAVAPI_engine_module_type
{
    /* AVE module type */
    SAVAPI_ENGINE_MODULE_AVE = 0,

    /* VDF module type */
    SAVAPI_ENGINE_MODULE_VDF
} SAVAPI_ENGINE_MODULE_TYPE;

/** @} */
/**
 * \defgroup typedefs SAVAPI typedefs
 * @{
 */

/**
 * \brief SAVAPI instance handle.
 */
typedef void * SAVAPI3_FD;

/**
 * \brief SAVAPI callback function pointer definition.
 * \param data [IN]: Pointer to the structure containing the callback data.
 */
typedef int(*SAVAPI3_CALLBACK)(SAVAPI3_CALLBACK_DATA *data);

/**
 * \brief SAVAPI callback for logging
 * \param log_level [IN]: The log level for the given message
 * \param message   [IN]: The message to be logged
 * \param user_data [IN]: The user context
 * \return Nothing
 */
typedef void(*SAVAPI3_LOG_CALLBACK)(SAVAPI3_LOG_LEVEL log_level, const SAVAPI_TCHAR *message, void *user_data);

/**
 * \brief Callback function used to return one engine single module
 * \param name      : The name of the module
 * \param type      : The type of the returned module
 * \param user_data : Pointer to the user-specific data
 */
typedef int (*SAVAPI_ENGINE_MODULE_CALLBACK)(const SAVAPI_TCHAR *name, SAVAPI_ENGINE_MODULE_TYPE type, void *user_data);
/** @} */

#ifdef __cplusplus
extern "C"
{
#endif


/**
 * \defgroup handle_funcs SAVAPI function pointers
 * \brief Handle types for exported SAVAPI functions
 * @{
 */

/**
 * @}
 * \ingroup handle_funcs
 * \defgroup handle_funcs_main SAVAPI main function pointers
 * \brief Handle types for exported SAVAPI main functions
 * @{
 */
typedef int (*SAVAPI3_set_log_callback_t)(SAVAPI3_LOG_CALLBACK log_fct, SAVAPI3_LOG_LEVEL min_level, void *user_data);
typedef int (*SAVAPI3_initialize_t)(SAVAPI3_GLOBAL_INIT *savapi_init);
typedef int (*SAVAPI3_uninitialize_t)();
typedef int (*SAVAPI3_get_version_t)(SAVAPI3_VERSION *version);
typedef int (*SAVAPI3_create_instance_t)(SAVAPI3_INSTANCE_INIT *init, SAVAPI3_FD *savapi_fd);
typedef int (*SAVAPI3_release_instance_t)(SAVAPI3_FD *savapi_fd);
typedef int (*SAVAPI3_set_user_data_t)(SAVAPI3_FD *savapi_fd, void *user_data);
typedef int (*SAVAPI3_is_running_ex_t)(const SAVAPI_TCHAR *hostname, unsigned int port);
typedef int (*SAVAPI3_register_callback_t)(SAVAPI3_FD *savapi_fd, unsigned int callback_id, SAVAPI3_CALLBACK callback);
typedef int (*SAVAPI3_unregister_callback_t)(SAVAPI3_FD *savapi_fd, unsigned int callback_id, SAVAPI3_CALLBACK callback);
typedef int (*SAVAPI3_scan_t)(SAVAPI3_FD *savapi_fd, SAVAPI_TCHAR *file_name);
typedef int (*SAVAPI3_set_t)(SAVAPI3_FD *savapi_fd, unsigned int option_id, SAVAPI_TCHAR *buffer);
typedef int (*SAVAPI3_get_t)(SAVAPI3_FD *savapi_fd, unsigned int option_id, SAVAPI_TCHAR *buffer, SAVAPI_SIZE_T *buffer_size);
typedef int (*SAVAPI3_send_signal_t)(SAVAPI3_FD *savapi_fd, unsigned int signal_id, SAVAPI3_SIGNAL_DATA* data);
typedef int (*SAVAPI3_set_fops_t)(SAVAPI3_FD *savapi_fd, void *fops_pointer, void *fops_context);
typedef void (*SAVAPI3_free_t)(void **ptr);
typedef int (*SAVAPI3_reload_engine_ex_t)(const SAVAPI3_GLOBAL_INIT *global_init);
typedef int (*SAVAPI3_extract_malware_names_t)(const SAVAPI_TCHAR *dir_path);
typedef int (*SAVAPI3_engine_modules_get_t)(const SAVAPI3_GLOBAL_INIT *init, SAVAPI_ENGINE_MODULE_CALLBACK module_func, void *user_data);

/**
 * @}
 * \defgroup funcs SAVAPI functions
 * @{
 */
/**
 * \brief Sets the SAVAPI logging function
 * \param log_fct   [IN]: The function used for logging. If given  function is NULL all data set with a previous
 *                        SAVAPI3_set_log_callback call will be cleared so logging will not be performed anymore.
 * \param min_level [IN]: Sets the desired minimum log level. This can be used to filter unwanted log-levels, so that if a
 *                        message have a lower level, it will be automatically "thrown" by the SAVAPI
 * \param user_data [IN]: The user context
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note This function can be called before or/and after global initialization (calling before it's recommended, so that
 *       any error messages in the SAVAPI3_initialize() can be logged)
 * \note This can be called several times in the same process, so that SAVAPI's user can change the log-level on-the-fly
 */
int SAVAPI3_EXP SAVAPI3_set_log_callback(SAVAPI3_LOG_CALLBACK log_fct, SAVAPI3_LOG_LEVEL min_level, void *user_data);

/**
 * \brief SAVAPI initialization function
 * Initializes the SAVAPI library, according to the parameters specified in the initialization structure.
 * It should be called once per process.
 * \param savapi_init [IN]: A pointer to the initialization structure, which must be filled with the proper values for initialization
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note The initialization function must be called a single time per process and before calling any other
 *       SAVAPI function (except the SAVAPI3_set_log_callback() function).
 */
int SAVAPI3_EXP SAVAPI3_initialize(SAVAPI3_GLOBAL_INIT *savapi_init);

/**
 * \brief SAVAPI uninitialization function
 * Uninitializes the SAVAPI library, cleaning up all used resources. Once called, all subsequent SAVAPI calls will fail
 * with SAVAPI3_E_NOT_INITIALIZED error code.
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note The uninitialization function must be called a single time per process and must be last called
 *       SAVAPI function.
 * \note All SAVAPI instances must be released before calling this function, otherwise the \ref SAVAPI3_E_BUSY will be returned.
 */
int SAVAPI3_EXP SAVAPI3_uninitialize();

/**
 * \brief Returns SAVAPI version
 *
 * \param version [OUT]: Pointer to the structure where to store the result
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note If this function is called before \ref SAVAPI3_initialize(),
 *       the \ref SAVAPI3_E_NOT_INITIALIZED will be returned instead.
 */
int SAVAPI3_EXP SAVAPI3_get_version(SAVAPI3_VERSION *version);

/**
 * \brief SAVAPI factory function
 * The function opens a connection to the SAVAPI daemon for client-mode, or,
 * for library mode, it creates a new SAVAPI instance.
 *
 * \param init       [IN]: Pointer to a structure containing all the initialization data needed to create a
 *                         SAVAPI instance.
 * \param savapi_fd [OUT]: Handle to the SAVAPI instance. To be used in all the subsequent SAVAPI calls
 * \return SAVAPI3_S_OK on success and an error otherwise
 */
int SAVAPI3_EXP SAVAPI3_create_instance(SAVAPI3_INSTANCE_INIT *init, SAVAPI3_FD *savapi_fd);

/**
 * \brief Destroys a SAVAPI handler, previously created with \ref SAVAPI3_create_instance.
 * The function closes the connection to the SAVAPI daemon for client-mode, or,
 * for library mode, it releases the SAVAPI instance.
 *
 * \param savapi_fd [IN/OUT]: SAVAPI instance to be released. As a precaution, the  pointer will be
 *                            nulled in order to become very clear that pointer will be unusable for now on
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note \li For each handler created with \ref SAVAPI3_create_instance function, the
 *       correspondent \c \b SAVAPI3_release_instance must be called!
 *       \li After calling the function the \c \b savapi_fd pointer will be invalid and must not be used anymore
 */
int SAVAPI3_EXP SAVAPI3_release_instance(SAVAPI3_FD *savapi_fd);

/**
 * \brief Sets user specific data.
 * This functions sets user data that will be returned untouched as <i> user_data </i>
 * member of \ref SAVAPI3_CALLBACK_DATA structure.
 *
 * \param savapi_fd [IN/OUT]: Handle to the SAVAPI instance.
 * \param user_data     [IN]: user specific data
 * \return SAVAPI3_S_OK in case of success or an error code otherwise
 *
 * \note The user is responsible with the memory management. This function will only set the value given in the
 *       \ref SAVAPI3_CALLBACK_DATA structure. It will not reserve or free memory for the data.
 */
int SAVAPI3_EXP SAVAPI3_set_user_data(SAVAPI3_FD *savapi_fd, void *user_data);

/**
 * \brief Determines if the SAVAPI daemon is running
 * The library must be initialized with the proper daemon connection parameters
 * for this function to run correctly.
 *
 * \return
 * - 0 If the daemon is stopped
 * - 1 If the daemon is running
 * - SAVAPI3_E_NOT_INITIALIZED If the SAVAPI library was not properly initialized.
 *
 * \note This function is deprecated, use \ref SAVAPI3_is_running_ex() instead.
 */
int SAVAPI3_EXP SAVAPI3_is_running();

/**
 * \brief Determines if the SAVAPI daemon is running on the specified interface (hostname and port)
 * \param hostname [IN]: Specifies the host on which the SAVAPI daemon is located.
 * \param port     [IN]: Specifies the port on which to connect to the daemon.
 * \return SAVAPI3_S_OK if the SAVAPI daemon is running on the given interface or an error code otherwise.
 *
 * \note For local sockets (see \ref SAVAPI3_FLAG_USE_LOCAL_SOCKET) the \param hostname must be a
 *       path to a file and the \param port must be 0.
 * \note This function returns \ref SAVAPI3_E_NOT_SUPPORTED in library mode.
 */
int SAVAPI3_EXP SAVAPI3_is_running_ex(const SAVAPI_TCHAR *hostname, unsigned int port);

/**
 * \brief Registers a client defined callback.
 * \param savapi_fd   [IN]: Handle to the SAVAPI instance on which the callback will be available.
 * \param callback_id [IN]: The callback type (e.g. SAVAPI3_CALLBACK_REPORT_FILE_STATUS, SAVAPI3_CALLBACK_ARCHIVE_OPEN etc.)
 * \param callback    [IN]: Pointer to a callback function
 * \return SAVAPI3_S_OK if everything went OK or an error code otherwise\n\n
 *
 * \note Callback registering is not allowed during scanning operations, otherwise the \ref SAVAPI3_E_BUSY will be returned.
 * \note Only one callback function is allowed to be registered per callback type/id. If for the given type/id
 *       a callback function was already registered then this function will return \ref SAVAPI3_E_INVALID_PARAMETER
 *      \sa callbacks_id
 */
int SAVAPI3_EXP SAVAPI3_register_callback(SAVAPI3_FD *savapi_fd, unsigned int callback_id, SAVAPI3_CALLBACK callback);

/**
 * \brief Unregisters a previously registered client defined callback
 * \param savapi_fd   [IN]: Handle to the SAVAPI instance.
 * \param callback_id [IN]: The callback type (e.g. SAVAPI3_CALLBACK_REPORT_FILE_STATUS, SAVAPI3_CALLBACK_ARCHIVE_OPEN etc.)
 * \param callback    [IN]: Pointer to a callback function
 * \return SAVAPI3_S_OK in case of success or an error code otherwise
 *
 * \note Callback unregistering is not allowed during scanning operations, otherwise the \ref SAVAPI3_E_BUSY will be returned.
 * \note The callback type/id is searched by SAVAPI in its internal callback list and if found then the callback function
 *       will be removed, otherwise this function will return \ref SAVAPI3_E_INVALID_PARAMETER
 */
int SAVAPI3_EXP SAVAPI3_unregister_callback(SAVAPI3_FD *savapi_fd, unsigned int callback_id, SAVAPI3_CALLBACK callback);

/**
 * \brief Starts a scanning process. During the scan operation the registered callbacks may be triggered.
 * \param savapi_fd [IN]: The handle of the SAVAPI instance that will do the scanning
 * \param file_name [IN]: The name of the file to be scanned.
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note The execution will not leave \ref SAVAPI3_scan function until scan process is finished.
 * \note SAVAPI supports various scan types depending on the file_name format:
 *       - 'path/to/the/file/on/disk' for normal file scanning.
 *       - 'mem://0xAddress,size,name' for scan in memory.
 *         The '0xAddress' is the memory area where the buffer with size 'size' is loaded.
 *         The 'name' is the display name used when callbacks are triggered.
 *       - 'hex_enc://hex_encoded_filename' for scanning files with filename given using hex encoding.
 *         This is useful for special encodings (ex. Chinese) in order to avoid conversions.
 *         This is not available on Windows platforms and in client-mode: SAVAPI3_E_NOT_SUPPORTED will be returned
 *         All filenames returned via callbacks will also be hex-encoded.
 */
int SAVAPI3_EXP SAVAPI3_scan(SAVAPI3_FD *savapi_fd, SAVAPI_TCHAR *file_name);

/**
 * \brief Sets SAVAPI individual settings.
 * \param savapi_fd [IN]: Pointer to a SAVAPI instance
 * \param option_id [IN]: The id of the option to be set
 * \param buffer    [IN]: NULL-terminated string containing the value of the option to be set
 *
 * \return SAVAPI3_S_OK If everything went ok an error code otherwise.
 * \note Calling this function during a scanning operation performed by this instance will fail.
 */
int SAVAPI3_EXP SAVAPI3_set(SAVAPI3_FD *savapi_fd, unsigned int option_id, SAVAPI_TCHAR *buffer);

/**
 * \brief Reads SAVAPI settings.
 * \param savapi_fd       [IN]: Pointer to a SAVAPI instance
 * \param option_id       [IN]: The id of the option to be retrieved
 * \param buffer         [OUT]: Buffer allocated by caller which will store the result of a successful get as a NULL terminated string.
 *                              If the buffer is NULL and the other parameters are valid, the function will set the needed buffer-size and return SAVAPI3_S_OK
 * \param buffer_size [IN/OUT]: Specifies the size, given in SAVAPI_TCHAR characters, of the buffer argument.
 *                              If the buffer is not large enough, upon return it will contain the needed size. If it's
 *                              equal or larger than the needed size, it will remain unchanged.
 *
 * \return SAVAPI3_S_OK on success and an error otherwise
 */
int SAVAPI3_EXP SAVAPI3_get(SAVAPI3_FD *savapi_fd, unsigned int option_id, SAVAPI_TCHAR *buffer, SAVAPI_SIZE_T *buffer_size);

/**
 * \brief Retrieve the various types that can be detected (and dynamically turned on/off).
 * \param type [IN]: The type that should be detected (current values: ADSPY,BDC,DIAL,GAME,HEUR_DBLEXT,JOKE,PCK,SPR)
 * \param id  [OUT]: Stores the type id
 * \return SAVAPI3_S_OK on success and an error otherwise
 * \warning Not implemented yet.
*/
int SAVAPI3_EXP SAVAPI3_get_dynamic_detect(SAVAPI_TCHAR *type, int *id);

/**
 * \brief Sends a signal to a specific SAVAPI instance
 * The \ref SAVAPI3_scan may take a long amount of time to finish scanning its target and in some
 * situations a forced abort would be desirable. In these kind of situations, SAVAPI3_send_signal may help
 * by sending signals to a running SAVAPI instance (\ref SAVAPI3_SIGNAL_SCAN_ABORT for instance).
 *
 * \param savapi_fd [IN]: SAVAPI instance
 * \param signal_id [IN]: Identifies the signal to be sent. See \ref signal_ids "Signal IDs section"
 * \param data      [IN]: Specific data to be sent when sending the signal. See \ref SAVAPI3_SIGNAL_DATA
 * \return SAVAPI3_S_OK for success or an error code otherwise.
 *
 * \note The SAVAPI signals were designed to be sent asynchronously, when an event arrives (Ctrl+C was issued
 *       for instance) and if program execution is within \ref SAVAPI3_scan. It makes no sense to
 *       send a signal to a SAVAPI instance when we have execution flow control.
 *       \sa Current supported \ref signal_ids
 */
int SAVAPI3_EXP SAVAPI3_send_signal(SAVAPI3_FD *savapi_fd, unsigned int signal_id, SAVAPI3_SIGNAL_DATA* data);

/**
 * \brief Specify the new fops who will be used by the engine for reading
 * \param savapi_fd    [IN]: SAVAPI instance
 * \param fops_pointer [IN]: Pointer to the fops to use in the current savapi session
 * \param fops_context [IN]: This context will be passed back to the application by each call to the fops used
 * \return SAVAPI3_S_OK for success or an error code otherwise.
 *
 * \note This function will return \ref SAVAPI3_E_NOT_SUPPORTED in client-mode
 */
int SAVAPI3_EXP SAVAPI3_set_fops(SAVAPI3_FD *savapi_fd, void *fops_pointer, void *fops_context);

/**
 * \brief Frees the memory space pointed to by ptr
 * \param ptr [IN/OUT]: Pointer who will become free and null
 * \return Nothing
 */
void SAVAPI3_EXP SAVAPI3_free(void **ptr);

/**
 * \brief Reloads the engine from the location given at global initialization
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note This function will simply call the \ref SAVAPI3_uninitialize routine
 *       followed by the \ref SAVAPI3_initialize having as parameter the same
 *       data provided at global initialization (aka the call to \ref
 *       SAVAPI3_initialize) in order to (re)load the engine files from the
 *       initial location(s).
 * \note All constraints that apply to the \ref SAVAPI3_uninitialize and
 *       \ref SAVAPI3_initialize are also available for this function. Therefore,
 *       in order to call the function, all instances must be released otherwise
 *       the \ref SAVAPI3_E_BUSY will be returned.
 * \note For better functionality and the possibility to load a new engine without interrupt
 *       of service, please check the \ref SAVAPI3_reload_engine_ex function.
 * \note This function will return \ref SAVAPI3_E_NOT_SUPPORTED in client-mode
 * \warning If something wrong goes when the new engine is loaded (e.g. engine is
 *       corrupted, some engine files are missing, etc), the library will not be
 *       usable anymore (i.e. all functions will return \ref SAVAPI3_E_NOT_INITIALIZED)
 *       so the user should only call \ref SAVAPI3_uninitialize and abort the execution.
 */
int SAVAPI3_EXP SAVAPI3_reload_engine();

/**
 * \brief Reloads the engine from the specified location
 * \param global_init [IN]: A pointer to the initialization structure containing the paths
 *                          to the new engine and vdf files
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note When this function is called, the engine will be (re)loaded from
 *       the specified path. The old engine's instances will be kept until
 *       their reference counter will reach 0. Calling this function, affects
 *       only the new SAVAPI instances (obtained by calling SAVAPI3_create_instance()).
 *       They will use the new loaded engine. The SAVAPI instances that are
 *       already started won't be affected by this function, they will continue
 *       to use the engine that they were started with.
 * \note This function will return \ref SAVAPI3_E_NOT_SUPPORTED in client-mode
 */
int SAVAPI3_EXP SAVAPI3_reload_engine_ex(const SAVAPI3_GLOBAL_INIT *global_init);

/**
 * \brief Extracts the malware names from memory to disk.
 *        The purpose of the function is to reduce the amount of memory (RAM) used
 *        by the SAVAPI Library. The basic idea is to unload the malware names
 *        (which are only needed in case of alerts) from memory and dump them to disk.
 * \param dir_path [IN]: The directory where the file containing the malware names
 *                       information will be created. If set to NULL, the function
 *                       will use the system temporary folder.
 *
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note The function will create, in the chosen folder (the \a dir_path or the system
 *       temporary folder), a file with the following naming scheme:
 *       'AV-malware-names-<process-PID>-<6 random chars>'. In order to
 *       retrieve the path to the file, the \ref SAVAPI3_get function along with
 *       the \ref SAVAPI3_OPTION_MALWARE_NAMES_FILE option should be used.
 *       The file will be kept opened by the SAVAPI Library as long as the current engine
 *       is in use and will be closed and removed afterwards (i.e. after a successfully
 *       call to one of \ref SAVAPI3_reload_engine, \ref SAVAPI3_reload_engine_ex and
 *       \ref SAVAPI3_uninitialize functions). Thus after a call to the \ref SAVAPI3_reload_engine
 *       or \ref SAVAPI3_reload_engine_ex, this function should be called again in
 *       order to extract the new engine's malware names.
 * \note It has to be ensured that the Library has the appropriate permissions to create
 *       the file and that there is enough free disk space - around 100 Mb.
 * \note This function will return \ref SAVAPI3_E_NOT_SUPPORTED in client-mode.
 * \note The function is available starting with SAVAPI Library 1.4.0.1 version.
 * \warning Calling this function more than once for a single engine is forbidden
 *          and the \ref SAVAPI3_E_BUSY error will be returned instead!
 */
int SAVAPI3_EXP SAVAPI3_extract_malware_names(const SAVAPI_TCHAR *dir_path);

/**
 * \brief Retrieves the engine components through the provided callback
 * \param init        [IN]: A pointer to the initialization structure
 * \param module_func [IN]: Pointer to the function to be called for every engine module
 * \param user_data   [IN]: Pointer to the user-defined data
 * \return SAVAPI3_S_OK on success and an error otherwise
 *
 * \note Function is available starting with SAVAPI Library versions greater than 1.4.0.18
 * \note This function will return \ref SAVAPI3_E_NOT_SUPPORTED in client-mode.
 */
int SAVAPI3_EXP SAVAPI3_engine_modules_get(const SAVAPI3_GLOBAL_INIT *init, SAVAPI_ENGINE_MODULE_CALLBACK module_func, void *user_data);
/** @} */
#ifdef __cplusplus
}
#endif
#endif /* SAVAPI_H__ */
