#!/usr/bin/env bash
#
# Copyright (c) 2012 Avira Operations GmbH & Co. KG
#
#
# Description: Utils functions used for Avira ICAP update with different engine directory
# VERSION=1.0.0.2

# This script must be executed before an Avira ICAP update for preparing the configuration and bash files when 
# the engine directory is different than the Avira ICAP install directory.

# The inputs of the script are Avira ICAP install and engine directory.
# Usage: ./script.sh <Avira_ICAP_install_dir> <engine_dir>
# The script returns a message about the modified and backuped files.

# This script:
# - creates backuped files for icap_post_utils.sh, avupdate-icap-product.conf, avupdate-icap-engine.conf and icap_update.sh;
# - changes the value of ICAPBIN_DIR with the Avira ICAP install dir;
# - creates symlinks in engine directory for icap_pre.sh, icap_post.sh and icap_post_utils.sh;
# - changes the avupdate configuration files with the engine directory and update modules list; 
# - adds command for updating the engine files in icap_update.sh.
set -e

check_directory(){
        
	local dir="$1"
        
	# check if the directory exists
	if [ ! -d $dir ]; then
        echo "The directory $dir does not exist."
        return 1
	fi
     
	# check if the directory has read, write and execute permissions
	if [ ! -r $dir ]; then
        echo "The directory $dir does not have read permissions"
        return 1
	fi
	
	if [ ! -w $dir ]; then
		echo "The directory $dir does not have write permissions"
        return 1
	fi 

	if [ ! -x $dir ]; then
		echo "The directory $dir does not have execute permissions"
        return 1
	fi	
        
	return 0
}

check_files(){

	local check_exec=$1
	shift
	local list_of_files=$@

	for file in ${list_of_files[@]}; do
		# check if the file exists
		if [ ! -f "$file" ]; then
            echo "The file $file does not exist."
	        return 1
		fi

		# check if the file has read, write and execute permissions
		if [ ! -r "$file" ]; then
	        echo "The file $file does not have read permissions"
	        return 1
		fi		
		
		if [ ! -w "$file" ]; then
			echo "The file $file does not have write permissions"
	        return 1
		fi

		if [ ! -x "$file" ] && $check_exec; then
			echo "The file $file does not have execute permissions"
	        return 1
		fi
	done

	return 0
}

create_backup(){

	local list_of_files=$@

	# create the backup files
	for file in ${list_of_files[@]}; do
		cp "$file" "$file.backup"
	done
	
	return 0
}

copy_script_files(){

	local engine="$1"
	shift
	local list_of_files=$@

	# create symlink
	for file in ${list_of_files[@]}; do 
		ln -s "$file" $engine
	done

	return 0
}

modify_utils_file(){

	local utils_file_path="$1"

	# uncomment the line with ICAPBIN_DIR
	sed -i 's/^#ICAPBIN_DIR/ICAPBIN_DIR/g' $utils_file_path

	# modify the value for ICAPBIN_DIR with the install directory
	sed -i "s|ICAPBIN_DIR=[^ ]*|ICAPBIN_DIR='$install_dir'|g" $utils_file_path
	
	return 0
}

modify_conf_file(){

	local engine_conf_file_path="$1"
	local product_conf_file_path="$2"
	local icap_update_file_path="$3"
	local engine_dir="$4"
	
	# modify the install din from engine configuration file
	sed -i "s|install-dir=[^ ]*|install-dir=$engine_dir|g" $engine_conf_file_path

	# modify the update modules list from product configuration file
	sed -i "s|^#update-modules-list|update-modules-list|g" $product_conf_file_path

	# add a new line before update product
	sed -i "s/# Update all files/\.\/avupdate\.bin --config=avupdate-icap-engine\.conf/" $icap_update_file_path

	return 0
}


# Main 

if [ "$1" == "--help" -o "$1" == "-h" ]; then
    echo "Usage: ./engine_dir_helper.sh <Avira_ICAP_install_dir> <engine_dir>"
    exit 1
fi

if [ $# -ne 2 ]; then
	echo "Incorrect, check '--help' or '-h'"
    exit 1
fi

# check if the install and engine directory exists and has read, write and execution rights
check_directory "$1"
check_directory "$2"

# read absolute paths of install and engine directories 
install_dir=$(readlink -e "$1")
engine_dir=$(readlink -e "$2")

# remove "/" from end of install directory
install_dir=${install_dir%/}

icap_post_utils_file_path=$install_dir"/icap_post_utils.sh"
icap_post_file_path=$install_dir"/icap_post.sh"
icap_pre_file_path=$install_dir"/icap_pre.sh"
icap_update_file_path=$install_dir"/icap_update.sh"

engine_conf_file_path=$install_dir"/avupdate-icap-engine.conf"
product_conf_file_path=$install_dir"/avupdate-icap-product.conf"

# list of update script files
list_of_bash_files=($icap_post_utils_file_path $icap_pre_file_path $icap_post_file_path $icap_update_file_path)

# list of update configuration files
list_of_conf_files=($engine_conf_file_path $product_conf_file_path)

# list of files for backup
list_of_file_backup=("$icap_post_utils_file_path" "$icap_update_file_path" "$engine_conf_file_path" "$product_conf_file_path")

# list of file for copy
list_of_file_for_copy=($icap_post_utils_file_path $icap_pre_file_path $icap_post_file_path)

# check if the script files exists and has read, write and execution rights
check_files true ${list_of_bash_files[@]}

# check if the configuration files exists and has read and write rights
check_files false ${list_of_conf_files[@]}

# create backup files
create_backup ${list_of_file_backup[@]}

# copy the script files to engine directory
copy_script_files $engine_dir ${list_of_file_for_copy[@]}

# modify the icap_post_utils file
modify_utils_file $icap_post_utils_file_path
	
# modify the configuration files
modify_conf_file $engine_conf_file_path $product_conf_file_path $icap_update_file_path $engine_dir

echo "INFO: The files ${list_of_file_backup[@]} were modified and backed-up in $install_dir."

